<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class LayoutImportacaoController extends CrudWithParameterBag {
    
    protected $dataSourceName = 'select_vallayout_imp'; 
    protected $util;
    protected $layImpService;
    protected $impConsumidorProcService;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag,  $util, Services\LayoutImportacaoService $layImpService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this -> util = $util;
        $this->layImpService = $layImpService;
    }
    
     
    public function findLayouts (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $CDFILIAL = isset($conditions['CDFILIAL']) && $conditions['CDFILIAL'] != "" ? $conditions['CDFILIAL'] : null;
        $layouts = $this->layImpService->findLayouts($CDFILIAL);
        
        $response->addDataSet(new DataSet('lay_imp_buscar_layouts', $layouts));
    }
    
    public function saveLayout (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        if ($conditions['__is_new']) {
            $this->layImpService->addLayout($conditions);
        } else {
            $this->layImpService->updateLayout($conditions);
        }
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteLayout (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->deleteLayout($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function findArquivos (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $CDLAYOUTIMP = isset($conditions['CDLAYOUTIMP']) && $conditions['CDLAYOUTIMP'] != "" ? $conditions['CDLAYOUTIMP'] : null;
        $arquivos = $this->layImpService->findArquivos($CDLAYOUTIMP);
        
        $response->addDataSet(new DataSet('lay_imp_buscar_arquivos', $arquivos));
    }
    
    public function saveConfig (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        if ($conditions['__is_new']) {
            $this->layImpService->addConfig($conditions);
        } else {
            $this->layImpService->updateConfig($conditions);
        }
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteConfig (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->deleteConfig($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function findFuncoes (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $arquivos = [];
        if (isset($conditions['CDLAYOUTIMP'])) {
            $arquivos = $this->layImpService->findFuncoes($conditions);
        }
        
        $response->addDataSet(new DataSet('lay_imp_buscar_funcoes', $arquivos));
    }
    
    public function saveFuncao (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        if ($conditions['__is_new']) {
            $this->layImpService->addFuncao($conditions);
        } else {
            $this->layImpService->updateFuncao($conditions);
        }
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteFuncao (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->deleteFuncao($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function findColunas (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $colunas = [];
        if (isset($conditions['CDLAYOUTIMP'])) {
            $colunas = $this->layImpService->findColunas($conditions);
        }
        
        $response->addDataSet(new DataSet('lay_imp_buscar_colunas', $colunas));
    }
    
    public function saveColuna (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        if ($conditions['__is_new']) {
            $this->layImpService->addColuna($conditions);
        } else {
            $this->layImpService->updateColuna($conditions);
        }
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteColuna (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->deleteColuna($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteMultiplasColunas (DTO\Request\DataSet $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);

        $this->layImpService->deleteMultiplasColunas($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function findColunasNaoCadastradas(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);

        $colunas = [];
        if (isset($conditions['CDLAYOUTIMP'])) {
            $colunas = $this->layImpService->findColunasNaoCadastradas($conditions);
        }
        
        $response->addDataSet(new DataSet('lay_imp_buscar_colunas_cadastro', $colunas));
    }
    
    public function findFormatos(DTO\Request\Filter $request, DTO\Response $response){
        $formatos = $this->layImpService->findFormatos();
        
        $response->addDataSet(new DataSet('lay_imp_buscar_colunas_cadastro', $formatos));
    }
    
    public function findValidacoes (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);

        $formatos = [];
        if (isset($conditions['CDLAYOUTIMP'])) {
            $formatos = $this->layImpService->findValidacoes($conditions);
        }
        
        $response->addDataSet(new DataSet('select_vallayout_imp', $formatos));
    }
    
    public function saveValidacao (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        if ($conditions['__is_new']) {
            $this->layImpService->addValLayoutImp($conditions);
        } else {
            $this->layImpService->updateValLayoutImp($conditions);
        }
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function deleteValidacao (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->deleteValidacao($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    public function checkFuncaoConsumidor (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
    
        $hasConsumerFunction = true;
        if (isset($conditions['CDLAYOUTIMP'])) {
            $hasConsumerFunction = $this->layImpService->checkFuncaoConsumidor($conditions['CDLAYOUTIMP']);
        }
     
        $response->addDataSet(new DataSet('verifica_funcao_consumidor', array('hasConsumerFunction' => $hasConsumerFunction)));       
    }
    
    public function findLayoutData (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);

        $data = [];
        if (isset($conditions['CDLAYOUTIMP'])) {
            $data = $this->layImpService->findLayoutData($conditions['CDLAYOUTIMP']);
        }
        
        $response->addDataSet(new DataSet('select_dados_layout', $data));
    }
    
     public function selectClienteLayout (DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);

        $clientes = $this->layImpService->selectClienteLayout($conditions);
        
        $response->addDataSet(new DataSet('select_cdcliente_layout', $clientes));
    }
    
    public function saveLayoutImpAut (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        
        $this->layImpService->updateLayoutImp($conditions);
        
        $response->addMessage(new Message("Sucesso"));
    }
    
    
	//////////////////////////////////
	// EXPORTAÇÃO/IMPORTAÇÃO LAYOUT //
	//////////////////////////////////

    private function padRight($str, $length) {
        while (strlen($str) < $length) {
            $str .= ' ';
        }
        return $str;
    }


    public function exportLayout (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
        
        $CDLAYOUTIMP = $conditions['CDLAYOUTIMP'];
        $inLinha = 0;
        $linhas = [];

        // Tabela LAYOUTIMP - LI
        $layoutImp = $this->layImpService->getLayoutImp($CDLAYOUTIMP);
        if (!empty($layoutImp)) {
            foreach ($layoutImp as $row) {
                $stLinha = 'LI' .
                           $this->padRight($row['CDLAYOUTIMP'], 10) .
                           $this->padRight($row['NMLAYOUTIMP'], 50) .
                           $this->padRight($row['IDSEPARCAMPO'], 1) .
                           $this->padRight($row['IDCARACDELIM'], 1) .
                           $this->padRight($row['IDCARACDECIM'], 1) .
                           $this->padRight($row['IDCARACSTRIN'], 1) .
                           $this->padRight($row['IDSISTEMAIMP'], 5) .
                           $this->padRight($row['IDSISTEMORIG'], 5);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
        
        // Tabela TABLAYOUTIMP - TI
        $tabLayoutImp = $this->layImpService->getTabLayoutImp($CDLAYOUTIMP);
        if (!empty($tabLayoutImp)) {
            foreach ($tabLayoutImp as $row) {
                $stLinha = 'TI' .
                           $this->padRight($row['CDLAYOUTIMP'], 10) .
                           $this->padRight($row['NRSEQTABELAY'], 10) .
                           $this->padRight($row['NMTABELABD'], 30) .
                           $this->padRight($row['NRORDIMPLAY'], 5) .
                           $this->padRight($row['NMARQLAYIMP'], 250);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
        

    
        // Tabela FUNLAYOUTIMP - FI
        $funLayoutImp = $this->layImpService->getFunLayoutImp($CDLAYOUTIMP);
        if (!empty($funLayoutImp)) {
            foreach ($funLayoutImp as $row) {
                $stLinha = 'FI' .
                           $this->padRight($row['CDLAYOUTIMP'], 10) .
                           $this->padRight($row['NRSEQTABELAY'], 10) .
                           $this->padRight($row['IDFUNARQLAY'], 3) .
                           $this->padRight($row['IDINACONSARQIMP'], 1) .
                           $this->padRight($row['IDPARALISAIMPARQ'], 1) .
                           $this->padRight($row['IDREMOVEZEROESQ'], 1);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
    
        // Tabela FRMTCOLLAY - FC
        $frmtColLay = $this->layImpService->getFrmtColLay();
        if (!empty($frmtColLay)) {
            foreach ($frmtColLay as $row) {
                $stLinha = 'FC' .
                           $this->padRight($row['CDFRMTCOLLAY'], 3) .
                           $this->padRight($row['DSFRMTCOLLAY'], 50);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
    
        // Tabela COLLAYOUTIMP - CI
        $colLayoutImp = $this->layImpService->getColLayoutImp($CDLAYOUTIMP);
        if (!empty($colLayoutImp)) {
            foreach ($colLayoutImp as $row) {
                $stLinha = 'CI' .
                           $this->padRight($row['CDLAYOUTIMP'], 10) .
                           $this->padRight($row['NRSEQTABELAY'], 10) .
                           $this->padRight($row['IDFUNARQLAY'], 3) .
                           $this->padRight($row['NRSEQCOLULAY'], 10) .
                           $this->padRight($row['NRPOSICOLLAY'], 15) .
                           $this->padRight($row['NRTAMCOLLAY'], 15) .
                           $this->padRight($row['NRCASASDECIM'], 15) .
                           $this->padRight($row['DSDEFACOLLAY'], 50) .
                           $this->padRight($row['IDGERSEQCOLU'], 1) .
                           $this->padRight($row['IDTDADCOLLAY'], 1) .
                           $this->padRight($row['IDGERCODCOLU'], 1) .
                           $this->padRight($row['CDFRMTCOLLAY'], 3) .
                           $this->padRight($row['NMTABELAIMP'], 30) .
                           $this->padRight($row['NMCOLUNAIMP'], 30) .
                           $this->padRight($row['DSCONCATESQ'], 50) .
                           $this->padRight($row['DSCONCATDIR'], 50);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
    
        // Tabela VALLAYOUTIMP - VA
        $valLayoutImp = $this->layImpService->getValLayoutImp($CDLAYOUTIMP);
        if (!empty($valLayoutImp)) {
            foreach ($valLayoutImp as $row) {
                $stLinha = 'VA' .
                           $this->padRight($row['CDLAYOUTIMP'], 10) .
                           $this->padRight($row['NRSEQTABELAY'], 10) .
                           $this->padRight($row['NRSEQVALLAY'], 10) .
                           $this->padRight($row['NRPOSINIVALLAY'], 10) .
                           $this->padRight($row['NRTAMVALLAY'], 10) .
                           $this->padRight($row['DSVALVALLAY'], 300);
    
                $stLinha = rtrim($stLinha) . "\r";
                $inLinha++;
                $linhas[] = $stLinha;
            }
        }
        
        // Gerar o arquivo de exportação
        $conteudoArquivo = implode(PHP_EOL, $linhas);
        $conteudoArquivo = mb_convert_encoding($conteudoArquivo, 'UTF-8', 'auto');

        $dateTime = new \DateTime();
        $formatDateTime = $dateTime->format('dmY_His');
        $fileName = "LAYOUTIMP_EXPORT_" . $formatDateTime . ".txt";

        $resultado = [
            'fileName' => $fileName,
            'fileContent' => base64_encode($conteudoArquivo),
            'contentType' => 'text/plain'
        ];

        // Devolver o conteúdo do arquivo para o front-end como dataset
        $response->addDataSet(new DataSet("resultadoArquivo", $resultado));
    }
    
    
    public function importLayout(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
    
        // Decodifica o arquivo base64
        $decodedFile = base64_decode($conditions['ARQUIVOLAYOUT']);
        $lines = explode("\n", $decodedFile);
    
        // Variáveis para controle de mensagens de erro
        $result = ['error' => false, 'message' => ''];
    
        $this->layImpService->beginTransaction();
    
        try {
            $boArqOk = false;
            
            // Atualização das colunas
            $this->atualizaTabelaColunas();
    
            foreach ($lines as $line) {
                
                $line = trim($line); // Remove espaços em branco das extremidades
                if (empty($line)) {
                    continue; // Pula linhas vazias
                }
    
                if (!$boArqOk) {
                    if (substr($line, 0, 2) === 'LI') {
                        $boArqOk = true;
                    } else {
                        $result['error'] = true;
                        $result['message'] = 'O arquivo informado encontra-se inválido.';
                        throw new \Exception($result['message']);
                    }
                }
    
                if (substr($line, 0, 2) === 'LI') {
                    $this->layImpService->insertLayoutImp($line);
                } elseif (substr($line, 0, 2) === 'TI') {
                    $this->layImpService->insertTabLayoutImp($line);
                } elseif (substr($line, 0, 2) === 'FI') {
                    $this->layImpService->insertFunLayoutImp($line);
                } elseif (substr($line, 0, 2) === 'FC') {
                    $this->layImpService->insertFrmtColLay($line);
                } elseif (substr($line, 0, 2) === 'CI') {
                    $this->layImpService->insertColLayoutImpImport($line);
                } elseif (substr($line, 0, 2) === 'VA') {
                    $this->layImpService->insertValLayoutImp($line);
                }
            }
    
            $this->layImpService->commitTransaction();
            $result['message'] = 'Layout importado com sucesso.';
        } catch (\Exception $e) {
            $this->layImpService->rollBackTransaction();
            $result['error'] = true;
            $result['message'] = $e->getMessage();
        }
    
        $response->addDataSet(new DataSet("resultadoImportacao", $result));
    }
    
    private function atualizaTabelaColunas() {
        $colunaImpResults = $this->layImpService->adqColunaImp();

        if (empty($colunaImpResults)) {
            $tabelaImpResults = $this->layImpService->adqTabelaImp();
    
            foreach ($tabelaImpResults as $row) {
                $this->layImpService->updColunaS(strtoupper($row['NMTABELABD']));
                $this->layImpService->updColunaN(strtoupper($row['NMTABELABD']));
                $this->layImpService->insColuna(strtoupper($row['NMTABELABD']));
            }
        }
    }
    
    ///////////////////////
	// ATUALIZAR COLUNAS //
	///////////////////////

    
    public function atualizaColunas(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->util->getParamsAsArray($request);
        
        $nomeTabela = $conditions['NMTABELA'];
    
        // Inserir a tabela
        $this->layImpService->insTabela($nomeTabela);
    
        // Consultar a tabela
        $tabela = $this->layImpService->adqTabela($nomeTabela);
        
        if (empty($tabela)) {
            $response->addDataSet(new DataSet("resultadoImportacao", ['erro' => true, 'mensagem' => "Tabela {$nomeTabela} não encontrada."]));
            return;
        }
    
        // Atualizar colunas
        $this->layImpService->updColunaS($nomeTabela);
        $this->layImpService->updColunaN($nomeTabela);
    
        // Executar stored procedure
        $this->layImpService->insColuna($nomeTabela);
    
        $response->addDataSet(new DataSet("resultadoImportacao", ['erro' => false, 'mensagem' => "Colunas atualizadas com sucesso."]));
    }
}