<?php

namespace Teknisa\MAN\Services;

use \Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Libs\Util\Environment;
use Teknisa\MAN\Factories\ConsumoRealizadoFactory;
//use Teknisa\MAN\Services\NovoCodigo;
use Teknisa\Libs\Util\NewCode;
use Teknisa\MAN\Utilities;


class ConsumoRealizadoService {
    
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'E' para 'Lançamento de Entrada' */
    const ENTRADA       = 'E';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'S' para 'Lançamento de Saida'   */
    const SAIDA         = 'S';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'A' para 'Ajuste de Inventario'
     * e 'Implantacao de Estoque'
     */
    const AJUSTE_IMP    = 'A';
    
    public static $logMessages = array();
    
    protected $environment;
    protected $ConsumoRealizadoFactory;
    protected $newCode;
    protected $irregularProducts;
    
    public function __construct(EntityManager $entityManager, Environment $environment, ConsumoRealizadoFactory $ConsumoRealizadoFactory, NewCode $newCode ) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->ConsumoRealizadoFactory = $ConsumoRealizadoFactory;
        $this->newCode = $newCode;
    }

    public function findConsumoRealizado (array $conditions) {
        $params = $this->buildParamsFromConditions($conditions);
        return $this->ConsumoRealizadoFactory->findConsumoRealizado($params);
    }
    
    protected function buildParamsFromConditions(array $conditions) {
        $params = array();
        foreach($conditions as $condition) {
            $params[$condition["columnName"]] = $condition["value"];
        }
        return $params;
    }
    
    public function insertConsumoRealizado($row) {
        try {    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $cdcontador = "CONSUMOREAL".$row["CDFILIAL"];
            $row["NRCONSREAL"]  = $this->newCode->getCodeByTableName($cdcontador, $this->environment->getNrOrg(), 1, 10);
            $row["VRCONSUMO"] = 0;
            $row["VRTOTCONSUMO"] = 0;
            $insert = $this->ConsumoRealizadoFactory->insertConsumoRealizado($row);
            $this->entityManager->commit();
            return $insert;
                         
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        } 
    }
    
    public function updateConsumoRealizado($row) {
        try {    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $update = $this->ConsumoRealizadoFactory->updateConsumoRealizado($row);
            $this->entityManager->commit();
            return $update;
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function deleteConsumoRealizado($row) {
        try {    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $existeConsumo = $this->ConsumoRealizadoFactory->verificaConsumoRealizado($row);
            if(!$existeConsumo){
                throw new \Exception('Consumo Realizado não existe.');
            } else {
                $result = $this->ConsumoRealizadoFactory->deleteConsumoRealizado($row);
                $this->entityManager->commit();
                return $result;
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }
    
    public function retiradaEstoqueConsumoRealizado($consumos, $retirada){
        try{
            $irregularesTotais = array();
            $retorno;
            
            foreach ($consumos as $consumo) {
                $connection = $this->entityManager->getConnection();

                #starts transaction of current loop
                $connection->beginTransaction();
                
                 $existeConsumo = $this->ConsumoRealizadoFactory->verificaConsumoRealizado($consumo);
                 
                 if($existeConsumo){
                     throw new \Exception("Retirada selecionada já realizada por outro operador. Atualize a Tela.");
                 }
                
                $retirada["DTLANCESTQ"] = $consumo['DTCONSUMO'];
                
                $this->validaDataLancamento($consumo['CDFILIAL'], $retirada['DTLANCESTQ'], self::SAIDA, null, $consumo['DTCONSUMO']);
                $nrOrg = $this->environment->getNrOrg();
                
                $cdcontador = "LANCTOEST".$retirada["CDFILIAL"];
                $retirada["NRLANESTSAID"]  = $this->newCode->getCodeByTableName($cdcontador, $nrOrg, 1, 10);
                //$retirada["NRLANESTSAID"]  = $this->getNovoCodigo($cdcontador, $nrOrg);
                $retirada['IDTPLANCTO'] = '3'; //ID PADRAO PARA LANÇAMENTO DE RETIRADA DE ESTOQUE 
                
                $insertLantoEsto = $this->ConsumoRealizadoFactory->adqinsertlancto($retirada);

                array_push(self::$logMessages, "Consumo: ".$consumo['NMPRODUTO']."\n"."Retirada: ".$retirada["NRLANESTSAID"]."\n"."Data: ".$retirada["DTLANCESTQ"]."\n");
                
                if(!empty($insertLantoEsto)){
                    
                    $cdcontador = "RETIESTO".$retirada["CDFILIAL"];
                    $retirada["NRRETEST"]  = $this->newCode->getCodeByTableName($cdcontador, $nrOrg, 1, 10);
                    //$retirada["NRRETEST"]  = $this->getNovoCodigo($cdcontador, $nrOrg);
                    
                    $insertretiradaesto = $this->ConsumoRealizadoFactory->adqinsertretiradaesto($retirada);
                    
                    if(!empty($insertretiradaesto)){    
                        $retirada["DSLANCESTIT"] = "Retirada para Produto Acabado: ".$retirada["NRLANESTSAID"];
                        //$subProdutos = $this->ConsumoRealizadoFactory->getProdutoAcabadoProdutosComposicao($consumo);
                        $subProdutos = $this->ConsumoRealizadoFactory->adqComposicaoLocal($consumo); //composicao Local

                        if(is_array($subProdutos) && count($subProdutos) == 0 ){
                            $subProdutos = $this->ConsumoRealizadoFactory->adqComposicaoPadrao($consumo); //composicao Geral
                        }
                        
                        if(is_array($subProdutos) && count($subProdutos) == 0 ){  
                            $subProdutos = $this->ConsumoRealizadoFactory->adqProdutoEstoque($consumo); //Produto estoque
                        }
                        
                        if(!empty($subProdutos)){
                            $this->itemLancEst($subProdutos, $consumo, $retirada);
                        }
                    }
                }
                
                $irregulares = array();
                try{
                    $this->atualizaEstoque($retirada['CDFILIAL'], $retirada["NRLANESTSAID"]);
                    $connection->commit();
                }catch(\Exception $e){
                    if($e->getMessage() == 'A confirmação da operação faria com que a posição de estoque de alguns itens se tornasse irregular.'){
                        $lanctoestoq['CDFILIAL'] = $retirada['CDFILIAL'];
                        $lanctoestoq['NRLANCESTQ'] = $retirada["NRLANESTSAID"];
                        $lanctoestoq['NRORG'] = $this->environment->getNrOrgTrab();
                        $produtoIrregular = $this->retornaProdutosIrregulares($lanctoestoq);

                        $arrFiltrado = array_filter($subProdutos, function($subProd) use ($produtoIrregular){
                            return $subProd['CDPRODESTO'] == $produtoIrregular['CDPRODUTO'];
                        });
                        
                        $arrFiltrado = array_values($arrFiltrado); // reindexa o array filtrado
                        if(count($arrFiltrado) == 0){
                            $arrFiltrado = $this->ConsumoRealizadoFactory->getProductData($produtoIrregular['CDPRODUTO']);
                        }
                        
                        $produtoIrregular['NMPRODUTO'] = $arrFiltrado[0]['NMPRODUTO'];
                        $produtoIrregular['CDARVPROD'] = $arrFiltrado[0]['CDARVPROD'];
                        $produtoIrregular['PRODCONSU'] = $consumo['NMPRODUTO'];
                        $produtoIrregular['NRCONSREAL'] = $consumo['NRCONSREAL'];
                        array_push($irregulares, $produtoIrregular);
                        array_push($irregularesTotais, $produtoIrregular);
                        $this->entityManager->rollback();
                    }else{
                        throw $e;
                    }
                }
                if(count($irregulares) == 0)
                    $this->ConsumoRealizadoFactory->updateStatusConsumoRealizado($consumo, $retirada["NRLANESTSAID"]);
                    
            }
            
            if(count($irregularesTotais) > 0 ){
                $retorno['IRREGULARES'] = $irregularesTotais;
            }
            $retorno['LOG'] = self::$logMessages;;
            
            return $retorno;
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        } 
    }
    
    public function itemLancEst($subProdutos, $consumo, $retirada){
        try{
            foreach ($subProdutos as $subProduto) {
                $tipoProd = 'C'; //composicao
                $subProduto['DTCONSUMO'] = $consumo['DTCONSUMO'];
                
                $temSubProdto = $this->ConsumoRealizadoFactory->adqComposicaoLocal($subProduto); //composicao Local
                
                if(is_array($temSubProdto) && count($temSubProdto) == 0 ){
                    $temSubProdto = $this->ConsumoRealizadoFactory->adqComposicaoPadrao($subProduto); //composicao Geral
                }
                
                if(is_array($temSubProdto) && count($temSubProdto) == 0 ){  
                    $tipoProd = 'E'; //Estoque
                    $temSubProdto = $this->ConsumoRealizadoFactory->adqProdutoEstoque($subProduto); //Produto estoque
                }
                
                if(!empty($temSubProdto) && ($temSubProdto[0]['CDPRODUTO'] != $subProduto['CDPRODUTO'] ) && $tipoProd != 'E'){
                    $novoConsumo = $consumo;
                    $novoConsumo['QTCONSUMO'] = $consumo['QTCONSUMO'] * $subProduto['QTITEMCOMPOS'];
                    $this->itemLancEst($temSubProdto, $novoConsumo, $retirada);
                } else {
                    if($subProduto['PRODESTOCNTRESTOQ'] != 'N'){
                        $nrseqitemult = $this->ConsumoRealizadoFactory->getnrseqitem($retirada);
                        
                        if(!$nrseqitemult['MAX(NRSEQUITEM)']){
                            $subProduto["NRSEQUITEM"] = '0001';
                        } else {
                            $subProduto["NRSEQUITEM"] = str_pad(strval($nrseqitemult['MAX(NRSEQUITEM)']+1), 4, "0", STR_PAD_LEFT);
                        }
                        
                        array_push(self::$logMessages, "Numero Sequencial do Item: ".$subProduto["NRSEQUITEM"]." Codigo do produto: ".$subProduto["CDPRODUTO"].
                        " Nome do produto: ".$subProduto["NMPRODUTO"]." Quantidade do produto: ".($subProduto['QTITEMCOMPOS']*$consumo['QTCONSUMO'])."\n");
                        
                        if (!empty($consumo["DTCONSUMO"])) {
                            list($dd,$mm,$yy) = explode('/',$consumo["DTCONSUMO"]);
                			$date = sprintf('%s-%s-%s',$yy,$mm,$dd);
                			$newdate = strtotime ( '+1 day' , strtotime($date)) ;
                            $subProduto["DTPOSICAO"] = date('d/m/Y', $newdate);
                        }
                        $data = $this->ConsumoRealizadoFactory->adqmaxdtposest($subProduto);
                        
                        $subProduto['DTPOSICAO'] = $data[0]['DTPOSIESTQ'];
                        $subProduto['DTCUSTO'] = $consumo['DTCONSUMO']; //$subProduto["DTMOVIESTQ"];
                        $subProduto['CDFILIPROC'] = $subProduto['CDFILIAL'];
                        //$subProduto[':NRLANCESTQ'] = null;
                        
                        $data = $this->ConsumoRealizadoFactory->adqposestlanc($subProduto, $retirada);
                        
                        if (!empty($data)) {
                            $subProduto['VRUNILANCTO'] = $data[0]['VRCUSTOPROD'];
                        }
                        if(!isset($subProduto['VRUNILANCTO']))
                            $subProduto['VRUNILANCTO'] = 0;
                            
                        $cdprodmovi = $subProduto['CDPRODESTO'];  //$this->ConsumoRealizadoFactory->selectcdprodmovi($subProduto); esta função nao faz nenhum sentido
                        $subProduto['CDPRODMOVI'] = $cdprodmovi;
                        
                        $subProduto['QTLANCTOEST'] = $subProduto['QTITEMCOMPOS']*$consumo['QTCONSUMO'];
                        $subProduto["VRTOTLANCTO"] = $subProduto['QTLANCTOEST']*$subProduto['VRUNILANCTO'];
                        $subProduto["VRLANCTOEST"] = $subProduto['VRUNILANCTO'];
                        $subProduto["QTTOTLANCTO"] = $subProduto['QTLANCTOEST'];
                        $subProduto["VRLANCTOBRUT"] = $subProduto['VRUNILANCTO'];
                        
                        $this->ConsumoRealizadoFactory->adqinsertitemlancesto($subProduto, $retirada);
                    }
                }
            }
            
            
        } catch(\Exception $e) {
            //$this->entityManager->rollback();
	        throw $e;
        } 
    }
    
    public function validaDataLancamento($filial, $dtLancamento, $operacao, $sistema, $dtEmissao) {
        try {
            $organizacao = $this->environment->getNrOrg();

            $dadosProcessFilial = $this->ConsumoRealizadoFactory->retornaDadosProcessFilial($filial, $organizacao);

            $dataProcess = $dadosProcessFilial['DTPROCESSA'];

            $utDtProcessAnt = $dadosProcessFilial['IDLANCDTMEN'];
            $utDtProcessPos = $dadosProcessFilial['IDLANCDTMAI'];
            $utDtProcessAntFilial = $dadosProcessFilial['IDLANCDTMENFIL'];
            $utDtProcessPosFilial = $dadosProcessFilial['IDLANCDTMAIFIL'];

            $qtDiasLancAntEntrada = $dadosProcessFilial['NRDIASENTR'];
            $qtDiasLancAntEntradaFilial = $dadosProcessFilial['NRDIASENTRFIL'];
            $qtDiasLancPostEntrada = $dadosProcessFilial['NRDIASENTRPD'];
            $qtDiasLancPostEntradaFilial = $dadosProcessFilial['NRDIASENTRPDFIL'];

            $qtDiasLancAntSaida = $dadosProcessFilial['NRDIASSAID'];
            $qtDiasLancAntSaidaFilial = $dadosProcessFilial['NRDIASSAIDFIL'];
            $qtDiasLancPostSaida = $dadosProcessFilial['NRDIASSAIDPD'];
            $qtDiasLancPostSaidaFilial = $dadosProcessFilial['NRDIASSAIDPDFIL'];

            $qtDiasLancAntAjuste = $dadosProcessFilial['NRDIASAJUS'];
            $qtDiasLancPostAjuste = $dadosProcessFilial['NRDIASAJUSPD'];
            $qtDiasLancAntAjusteFilial = $dadosProcessFilial['NRDIASAJUSFIL'];
            $qtDiasLancPostAjusteFilial = $dadosProcessFilial['NRDIASAJUSPDFIL'];
            $dtLancamento = DateUtil::getDataDeString($dtLancamento, DateUtil::FORMATO_BRASILEIRO, true);
            $dtEmissao = DateUtil::getDataDeString($dtEmissao, DateUtil::FORMATO_BRASILEIRO, true);
            $dataProcess = DateUtil::getDataDeString(DateUtil::truncateString($dataProcess), DateUtil::FORMATO_BRASILEIRO, true);

            $isDtLancAfterDtProcess = DateUtil::comparaDatas($dtLancamento, $dataProcess, DateUtil::DEPOIS);
            $isEqual = DateUtil::comparaDatas($dtLancamento, $dataProcess, DateUtil::IGUAL);

            if ($utDtProcessAnt === 'N' && $utDtProcessAntFilial === 'N' && !($isDtLancAfterDtProcess || $isEqual))
                throw new \Exception ('Data de lançamento deve ser maior ou igual à data de processamento.');

            if ($utDtProcessPos === 'N' && $utDtProcessPosFilial === 'N' && !(!$isDtLancAfterDtProcess || $isEqual))
                throw new \Exception ('Data de lançamento deve ser menor ou igual à data de processamento.');

            switch ($operacao) {
                case self::ENTRADA :
                    $this->validaDataLancamentoEntrada($dtLancamento, $dataProcess,
                        $utDtProcessAnt, $qtDiasLancAntEntrada,
                        $utDtProcessPos, $qtDiasLancPostEntrada,
                        $utDtProcessAntFilial, $qtDiasLancAntEntradaFilial,
                        $utDtProcessPosFilial, $qtDiasLancPostEntradaFilial,
                        $dtEmissao, $isDtLancAfterDtProcess, $isEqual
                    );
                    break;
                case self::SAIDA :
                    if ($sistema <> 'FAI' || $sistema <> 'FAC' || $sistema <> 'FAZ')
                        $this->validaDataLancamentoSaida($dtLancamento, $dataProcess,
                            $utDtProcessAnt, $qtDiasLancAntSaida,
                            $utDtProcessPos, $qtDiasLancPostSaida,
                            $utDtProcessAntFilial, $qtDiasLancAntSaidaFilial,
                            $utDtProcessPosFilial, $qtDiasLancPostSaidaFilial,
                            $isDtLancAfterDtProcess, $isEqual
                        );
                    break;
                case self::AJUSTE_IMP :
                    $this->validaDataLancamentoAjuste($dtLancamento, $dataProcess,
                        $utDtProcessAnt, $qtDiasLancAntAjuste,
                        $utDtProcessPos, $qtDiasLancPostAjuste,
                        $utDtProcessAntFilial, $qtDiasLancAntAjusteFilial,
                        $utDtProcessPosFilial, $qtDiasLancPostAjusteFilial,
                        $isDtLancAfterDtProcess, $isEqual
                    );
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function validaDataLancamentoSaida(
        $dtLancamento, $dataProcess, $utDtProcessAnt, $qtDiasLancAntSaida, $utDtProcessPos,
        $qtDiasLancPostSaida, $utDtProcessAntFilial, $qtDiasLancAntSaidaFilial,
        $utDtProcessPosFilial, $qtDiasLancPostSaidaFilial, $isDtLancAfterDtProcess, $isEqual
    ){
        if (!$isDtLancAfterDtProcess) {
            if($utDtProcessAntFilial == 'S'){
                $this->validaPeriodoLancamento($utDtProcessAntFilial, $dtLancamento, $dataProcess, $qtDiasLancAntSaidaFilial);
            }else{
                $this->validaPeriodoLancamento($utDtProcessAnt, $dtLancamento, $dataProcess, $qtDiasLancAntSaida);
            }
        } else {
            if($utDtProcessAntFilial == 'S'){
                $this->validaPeriodoLancamento($utDtProcessPosFilial, $dtLancamento, $dataProcess, $qtDiasLancPostSaidaFilial, DateUtil::DEPOIS);
            }else{
                $this->validaPeriodoLancamento($utDtProcessPos, $dtLancamento, $dataProcess, $qtDiasLancPostSaida, DateUtil::DEPOIS);
            }
        }
    }
    
    private function validaPeriodoLancamento(
        $utDtProcessAntPos, \DateTime $dtLancamento, \DateTime $dataProcess,
        $quantDias, $tipoComparacao = DateUtil::ANTES
    ){
        if ($tipoComparacao === DateUtil::ANTES)
            $dataProcess = DateUtil::subtraiIntervalo($dataProcess, $quantDias);
        else
            $dataProcess = DateUtil::adicionaIntervalo($dataProcess, $quantDias);

        if ($utDtProcessAntPos === 'S' && DateUtil::comparaDatas($dtLancamento, $dataProcess, $tipoComparacao))
            throw new \Exception ('Lançamento com data em '.substr($dtLancamento, 0, -9).' fora do período permitido.');
    }

    public function atualizaEstoque(
        $cdfilial, $nrlancestq, $verifyIrregularities = true, $abortAtIrregularities = true, $seqItemCodes = null
    ) {
            
        try {   
            
            $connection = $this->entityManager->getConnection();

            #starts main, outer transaction
            $connection->beginTransaction();
            
            try {
                #starts second inner transaction
                $connection->beginTransaction();
                
                $nrorg = $this->environment->getNrOrgTrab();
        
                $lanctoestoq = $this->ConsumoRealizadoFactory->buscaLanctoestoq($cdfilial, $nrlancestq, $nrorg);
        
                #marks all items as appurtenant to stock update
                $this->atualizaTodosItensLancamentoEstoque($cdfilial, $lanctoestoq['NRLANCESTQ'], $nrorg);
        
                $dadosProcess = $this->ConsumoRealizadoFactory->retornaDadosProcessFilial($cdfilial, $nrorg);
        
                #separate dates
                $dtprocessa = $dadosProcess['DTPROCESSA'];
                $dtlancestq = $lanctoestoq['DTLANCESTQ'];
        
                #process dates
                $dtprocessa = $this->getDateObject($dtprocessa);
                $dtlancestq = $this->getDateObject($dtlancestq);
        
                #order dates
                $date = $this->getOrderedDates($dtprocessa, $dtlancestq);
                
                #updates stock
                $this->executaProcedureAtualizaEstoque($cdfilial, $lanctoestoq['CDFILIMOVI'], $nrlancestq, $date["INITIAL"], $date["FINAL"]);
                
                #concludes second inner transaction
                $connection->commit();
                
            } catch (\Exception $e) {
                #throws error to cancel outer transaction
                throw $e;
            }

            #Please note that two separate transactions are
            #required for the correct operation of the function
            #This second step, where the procedure is executed
            #again, only works if the first one has already
            #been commited.

           try {
                #starts second inner transaction
                $connection->beginTransaction();

                #checks flags
                if($verifyIrregularities){
                    if($abortAtIrregularities)
                        #verifies irregularities and throws exception if any is found
                        $this->verificaIrregularidades($lanctoestoq, $seqItemCodes);
                    else {
                        #verifies irregularities and updates items for them to be ignored if any is found
                        $this->atualizaItensIrregulares($lanctoestoq, $seqItemCodes);

                        #updates stock with new parametrizations
                        $this->executaProcedureAtualizaEstoque($cdfilial, $lanctoestoq['CDFILIMOVI'], $nrlancestq, $date["INITIAL"], $date["FINAL"]);
                    }
                }

                #concludes second inner transaction
                $connection->commit();
                
            } catch (\Exception $e) {
                #rollbacks operations in this transaction
                
                $irregularProductCodes = array();
                $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);
                
                $connection->rollback();

                #throws error to cancel outer transaction
                throw $e;
            }

            $irregularProductCodes = array();
            $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);
            

            #separates irregular product codes
            if($irregularProducts){    
                foreach ($irregularProducts as $product) {
                    $irregularProductCodes[] = $product['CDPRODUTO'];
                }
            }
            #concludes outer transaction and all operations
            $connection->commit();
            
        } catch (\Exception $e) {
            #rollbacks every operation
            $irregularProductCodes = array();
            $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);
            
            $connection->rollback();

            #throws exception for controller to catch
            throw $e;
        }

        #returns any existent irregular product codes
        return $irregularProductCodes;
    }
    
    private function atualizaTodosItensLancamentoEstoque($cdfilial, $nrlancestq, $nrorg) {
        #gets all items from the received stock launch
        $itlanctoest = $this->ConsumoRealizadoFactory->buscaItlanctoest($cdfilial, $nrlancestq, $nrorg, 'N');

        #updates every item
        if(!is_null($itlanctoest)){
                $this->ConsumoRealizadoFactory->atualizaItensLancamentoEstoque($cdfilial, $nrlancestq, $nrorg, 'S');
        }
    }
     
    public function getOrderedDates($date, $otherDate) {
        #compares and separates dates in prior and subsequent
        if($date < $otherDate) {
            $firstDate = $date->format('d/m/Y');
            $lastDate = $otherDate->format('d/m/Y');
        } else {
            $firstDate = $otherDate->format('d/m/Y');
            $lastDate = $date->format('d/m/Y');
        }

        return array("INITIAL" => $firstDate, "FINAL" => $lastDate);
    }
   
    public function executaProcedureAtualizaEstoque($cdfilial, $cdfilimovi, $nrlancto, $dtinicial, $dtfinal) {
        #creates procedure
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_ESTOQUE');

        #sets all parameters
        $procedure->addParam(new Param('P_FILIMOVI', Param::PARAM_INPUT, $cdfilimovi, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTO', Param::PARAM_INPUT, $nrlancto, Param::PARAM_TYPE_STR));

        #execute procedure
        $procedure->execute();
    }

    public function verificaIrregularidades($lanctoestoq, $seqItemCodes = null) {
        #checks for irregularities
        $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);

        #TODO Verify a better message
        if (!empty($irregularProducts))
            throw new \Exception ('A confirmação da operação faria com que a posição de estoque de alguns itens se tornasse irregular.');
    }

    public function retornaProdutosIrregulares($lanctoestoq, $seqItens = null) {
        $nrmovilote = $lanctoestoq['CDFILIAL'].$lanctoestoq['NRLANCESTQ'];

        $produtosIrregulares = $this->ConsumoRealizadoFactory->selectMovi(
            $nrmovilote, $lanctoestoq['NRORG']
        );

        if(!is_null($seqItens)) {
            $itlanctoest = $this->ConsumoRealizadoFactory->buscaItlanctoest(
                $lanctoestoq['CDFILIAL'], $lanctoestoq['NRLANCESTQ'],
                $lanctoestoq['NRORG']
            );

            $seqProds = array();
            foreach($itlanctoest as $item)
                if(in_array($item['NRSEQUITEM'], $seqItens))
                    $seqProds[] = $item['CDPRODUTO'];

            $produtosIrregularesInSeqItens = array();

            foreach ($produtosIrregulares as $produto){
                if(in_array($produto['CDPRODUTO'], $seqProds)){
                    $produtosIrregularesInSeqItens[] = $produto;
                }
            }
            return $produtosIrregularesInSeqItens;
        }

        /** @var \Estoque\Entities\Movilote $produtosIrregulares */
        return $produtosIrregulares;
    }
    
    public function atualizaItensIrregulares($lanctoestoq, $seqItemCodes = null) {
        #checks for irregularities
        $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);

        #if there's any
        if(!empty($irregularProducts)) {
            #separates irregular product codes
            $productCodes = array();
            foreach($irregularProducts as $product)
                $productCodes[] = $product['CDPRODUTO'];

            #gets possibly irregular items
            $itlanctoest = $this->ConsumoRealizadoFactory->buscaItlanctoest(
                $lanctoestoq['CDFILIAL'], $lanctoestoq['NRLANCESTQ'],
                $lanctoestoq['NRORG'], 'S'
            );

            if(!is_null($itlanctoest)) {
                $seqItemCodes = array();
                foreach($itlanctoest as $item)
                    #checks every item, searching for irregular products
                    if(in_array($item['CDPRODUTO'], $productCodes))
                        #separes item codes that are related to irregular products
                        $seqItemCodes[] = $item['NRSEQUITEM'];

                #updates irregular items
                $this->atualizaItensLancamentoEstoquePorNrSeq($itlanctoest, 'N', $seqItemCodes);
            }
        }
    }
    
    private function atualizaItensLancamentoEstoquePorNrSeq($itlanctoest, $idevolestoq = 'S', $seqItemCodes = null) {
        #iterates over every received item
        foreach ($itlanctoest as $item) {
            #checks whether item should be updated
            if(is_null($seqItemCodes) || in_array($item['NRSEQUITEM'], $seqItemCodes)) {
                #updates item and persists information
                $item['IDEVOLESTOQ'] = $idevolestoq;
                $this->entityManager->persist($item);
                $this->entityManager->flush($item);
                if($idevolestoq == 'S')
                    $this->atualizaProdlcestoq($item['CDFILIAL'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], $item['CDPRODUTO']);
            }
            if(is_null($item->getIdtipomovi())){
                $item['IDTIPOMOVI'] = '1';
                $this->entityManager->persist($item);
                $this->entityManager->flush($item);
            }
        }
    }
        
    public function getDateObject($dateString) {
        #checks whether date variable is already an object
        if(!is_object($dateString)) {
            #evaluates appropriate date format
            $dateFormat = (strlen($dateString) > 10) ? DateUtil::FORMATO_BRASILEIRO_DATAHORA : DateUtil::FORMATO_BRASILEIRO;

            #creates object
            $dateObject = DateUtil::getDataDeString($dateString, $dateFormat, true);
        } else
            $dateObject = $dateString;

        return $dateObject;
    }
    
    public function getNovoCodigo ($CDCONTADOR, $NRORG) {
	    return NovoCodigo::geraCodigo($this->entityManager->getConnection(), $CDCONTADOR, $NRORG, 10);
	}
    
    public static function moneyToDouble($value) {
		if (!!$value) {
			$value = preg_replace("/[^0-9,]/", "", $value);
			$value = str_replace ("," , "." , $value);			
		}
		else {
			$value = 0;
		}
		
		return floatval($value);
	}

    
}